/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla MathML Editor.
 *
 * The Initial Developer of the Original Code is
 * MacKichan Software, Inc.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Steve Swanson (steve.swanson@mackichan.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// mathmled.js - top level commands for the MathML editor

// string bundle object
var bundle = srGetStrBundle("chrome://mathmled/locale/mathmled.properties");

var wind;
var doc;
var editStyle;
var gPrefsService;
var gPrefs;
var caretState;

const nsIFilePicker = Components.interfaces.nsIFilePicker;

function getService(s,i)
{
  return( Components.classes[s].getService(i) );
}

function createInstance(s,i)
{
  return( Components.classes[s].createInstance(i) );
}


function init(e)
{
  wind = top.frames[0];
  doc  = wind.document;

  try {
    gPrefsService = getService("@mozilla.org/preferences-service;1",
                               Components.interfaces.nsIPrefService);
  }
  catch(ex) {
    dump("failed to get prefs service!\n");
  }
  gPrefs = gPrefsService.getBranch(null);

  // enable caret so we can edit
  caretState = gPrefs.getBoolPref("accessibility.browsewithcaret");
  gPrefs.setBoolPref("accessibility.browsewithcaret",true);

  // so we (re)enter this function on future loads
  var contentArea = document.getElementById("mml-frame");
  contentArea.addEventListener("load", init, true);

  // clean out whitespace
  deleteWSfromMath(doc);

  // turn on caret at start of document
  var s = wind.getSelection();
  var bodyNode = doc.getElementsByTagName("body").item(0);
  s.collapse(bodyNode,0);

  // add CSS to colorize for editing
  editStyle = doc.createElementNS("http://www.w3.org/1999/xhtml","style");
  var css = doc.createTextNode("math {color: red} [tempinput='true'] {color: green}");
  editStyle.appendChild(css);
  editStyle.setAttribute("type","text/css");

  var headNode = doc.getElementsByTagName("head").item(0);
  if (headNode)
    headNode.appendChild(editStyle);
  else {
    headNode = doc.createElementNS("http://www.w3.org/1999/xhtml","head");
    headNode.appendChild(editStyle);
    bodyNode.parentNode.insertBefore(headNode,bodyNode);
  }

  updateColorControls();

  // this worked for a while ...
  wind.focus();
}

function exit()
{
  // restore caret (this doesn't seem to work)
  gPrefs.setBoolPref("accessibility.browsewithcaret",caretState);
  gPrefsService.savePrefFile(null);  

  window.close();
}

function doNewCmd()
{
  var n = document.getElementById("mml-frame");
  var loadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
  n.webNavigation.loadURI("chrome://mathmled/locale/mathmled-sample.xml", loadFlags, null, null, null);

  wind.focus();
}

// used by viewZoomOverlay.js
function getMarkupDocumentViewer()
{
  var n = document.getElementById("mml-frame");
  return n.markupDocumentViewer;
}

function doOpenCmd()
{
  try {
    var fp = createInstance("@mozilla.org/filepicker;1",nsIFilePicker);
    fp.init(window, bundle.GetStringFromName("openFileTitle"), nsIFilePicker.modeOpen);
    fp.appendFilters(nsIFilePicker.filterAll | nsIFilePicker.filterXML);

    if (fp.show() == nsIFilePicker.returnOK) {
      var n = document.getElementById("mml-frame");
      var loadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
      n.webNavigation.loadURI(fp.fileURL.spec, loadFlags, null, null, null);
    }
  } catch (ex) {
    alert("Bad Open");
  }

  wind.focus();
}

function doOpenLocCmd()
{
  // can't use chrome://communicator/content/openLocation.xul because of hard-coded behavior
  var o = new Object();
  o.val = "";
  window.openDialog("chrome://mathmled/content/mathmledOpenLoc.xul","_blank","chrome,close,titlebar,modal",o);
  if (o.val.length > 0) {
    try {
      var n = document.getElementById("mml-frame");
      var loadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
      n.webNavigation.loadURI(o.val, loadFlags, null, null, null);
    } catch (ex) {
    }
  }

  wind.focus();
}

function doSaveCmd()
{
  try {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = createInstance("@mozilla.org/filepicker;1",nsIFilePicker);
    fp.init(window, bundle.GetStringFromName("saveFileTitle"), nsIFilePicker.modeSave);
    fp.appendFilters(nsIFilePicker.filterAll | nsIFilePicker.filterXML);

    var res = fp.show();
    if (res == nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace) {
      var tempLocalFile = null;
      var docURI = createInstance("@mozilla.org/network/standard-url;1",
                              Components.interfaces.nsIURI);
      docURI.spec = fp.fileURL.spec;
      
      if (docURI.schemeIs("file"))
      {
        var ioService = getService("@mozilla.org/network/io-service;1", Components.interfaces.nsIIOService);
        var gFileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
        localFile = gFileHandler.getFileFromURLSpec(fp.fileURL.spec);
      }
      var os = createInstance("@mozilla.org/network/file-output-stream;1",
                              Components.interfaces.nsIFileOutputStream);
      os.init(localFile,0x2A,0644,0);   // write, create, truncate
      var tmp = '<?xml version="1.0"?>\n';
      os.write(tmp,tmp.length);
      tmp = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1 plus MathML 2.0//EN" "http://www.w3.org/TR/MathML2/dtd/xhtml-math11-f.dtd">\n';
      os.write(tmp,tmp.length);

      // temporarily remove edit style
      var headNode = doc.getElementsByTagName("head").item(0);
      headNode.removeChild(editStyle);

      var docAsStr = getDocMarkup(doc);

      // put it back
      headNode.appendChild(editStyle);

      os.write(docAsStr,docAsStr.length);
      os.close();
    }
  } catch (ex) {
    alert("Bad save: " + ex);
  }
}

// can't use Editor dialog because it needs editorShell :-(
function doEditAttributesCmd()
{
  var s = wind.getSelection();
  if (s) {
    var n  = getselectednode(s);
    window.openDialog("chrome://mathmled/content/mathmledAttributes.xul","_blank","chrome,close,titlebar,modal,resizable=yes",n);
    window.focus();
    if (window.AdvancedEditOK)
    {
      // nothing to do.  Dialog modifies DOM
    }
  }
}

function copyValue()
{
  alert("not implemented");
}

function pasteValue()
{
  alert("not implemented");
}

function doDeleteCmd()
{
  var s = wind.getSelection();
  if (s)
    deleteselection();
}

function doBackSpaceCmd()
{
  var s = wind.getSelection();
  if (s)
    deleteleft();
}

function doSpaceCmd()
{
  var s = wind.getSelection();
  if (s)
    movespace();
}

function doTabCmd()
{
  var s = wind.getSelection();
  if (s)
    tabobject();
}

function doEnterCmd()
{
  var s = wind.getSelection();
  if (s)
    dosplit();
}

function notImplemented()
{
  alert("not implemented");
}

function doHelpCmd()
{
  window.open('chrome://mathmled/content/mathmledHelp.xhtml','mathmled_about','scrollbars=1,resizable=1,width=500,height=300');
}

function doSelectAllCmd()
{
  var s = wind.getSelection();
  var bodyNode = doc.getElementsByTagName("body").item(0);
  s.selectAllChildren(bodyNode);
}


function doSmallCmd()
{
  var s = wind.getSelection();
  if (s)
    sizesmall();
}

function doBigCmd()
{
  var s = wind.getSelection();
  if (s)
    sizebig();
}

function doNormalSizeCmd()
{
  var s = wind.getSelection();
  if (s)
    sizenormal();
}

function doNormalCmd()
{
  var s = wind.getSelection();
  if (s) {
    stylebold(false);
    styleitalic(false);
  }
}

function doBoldCmd()
{
  var s = wind.getSelection();
  if (s) {
    stylebold(true);
    styleitalic(false);
  }
}

function doItalicCmd()
{
  var s = wind.getSelection();
  if (s) {
    stylebold(false);
    styleitalic(true);
  }
}

function doBoldItalicCmd()
{
  var s = wind.getSelection();
  if (s) {
    stylebold(true);
    styleitalic(true);
  }
}

function doToggleBold()
{
  var s = wind.getSelection();
  if (s)
    togglebold();
}

function doToggleItalic()
{
  var s = wind.getSelection();
  if (s)
    toggleitalic();
}


// For talking with the ColorPicker
var gColorObj = new Object();

gColorObj.LastTextColor = "";
gColorObj.LastBackgroundColor = "";
gColorObj.LastHighlightColor = "";
gColorObj.Type = "";
gColorObj.PageColor = "";
gColorObj.TableColor = "";
gColorObj.CellColor = "";
gColorObj.BackgroundColor = "";

var gLastTextColor = "#000000";
var gLastBackgroundColor = "#ffffff";


function doColorCmd()
{
  var s = wind.getSelection();
  if (!s)
    return;

  // Launch the ColorPicker dialog
  window.openDialog("chrome://editor/content/EdColorPicker.xul", "_blank", "chrome,close,titlebar,modal", "", gColorObj);

  // User canceled the dialog
  if (gColorObj.Cancel)
    return;

  setcolor(gColorObj.TextColor);
  if (gColorObj.TextColor)
    gLastTextColor = gColorObj.TextColor;
  else
    gLastTextColor = "#000000";
  updateColorControls();
}

function doBackgroundColorCmd()
{
  var s = wind.getSelection();
  if (!s)
    return;

  // Launch the ColorPicker dialog
  window.openDialog("chrome://editor/content/EdColorPicker.xul", "_blank", "chrome,close,titlebar,modal", "", gColorObj);

  // User canceled the dialog
  if (gColorObj.Cancel)
    return;

  setbackgroundcolor(gColorObj.TextColor);
  if (gColorObj.TextColor)
    gLastBackgroundColor = gColorObj.TextColor;
  else
    gLastBackgroundColor = "#ffffff";
  updateColorControls();
}

// Doing the right thing - having the controls track the caret - is hard.
function updateColorControls()
{
  var button = document.getElementById("TextColorButton");
  if (button)
    button.setAttribute("style", "background-color:"+gLastTextColor);
  button = document.getElementById("BackgroundColorButton");
  if (button)
    button.setAttribute("style", "background-color:"+gLastBackgroundColor);
}


function doMathCmd()
{
  var s = wind.getSelection();
  if (s)
    insertmath();
}

function doTextCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.val = "";
    window.openDialog("chrome://mathmled/content/mathmledtext.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.val.length > 0)
      inserttag("mtext",o.val);
  }
}

function doFractionCmd()
{
  var s = wind.getSelection();
  if (s)
    insertfraction();
}

function doSubCmd()
{
  var s = wind.getSelection();
  if (s)
    insertsub();
}

function doSupCmd()
{
  var s = wind.getSelection();
  if (s)
    insertsup();
}

function doSubSupCmd()
{
  var s = wind.getSelection();
  if (s)
    insertsubsup();
}

function doTensorCmd()
{
  var s = wind.getSelection();
  if (s)
    inserttensor();
}

function doRadicalCmd()
{
  var s = wind.getSelection();
  if (s)
    insertsqrt();
}

function doRootCmd()
{
  var s = wind.getSelection();
  if (s)
    insertroot();
}

function doDisplayCmd()
{
  var s = wind.getSelection();
  if (s)
    insertdisplay();
}

function doIntCmd()
{
  var s = wind.getSelection();
  if (s)
    insertintegral();
}

function doDIntCmd()
{
  var s = wind.getSelection();
  if (s)
    insertdoubleintegral();
}

function doCIntCmd()
{
  var s = wind.getSelection();
  if (s)
    insertcontourintegral();
}

function doSumCmd()
{
  var s = wind.getSelection();
  if (s)
    insertsum();
}

function doProdCmd()
{
  var s = wind.getSelection();
  if (s)
    insertprod();
}

function doOperatorCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.val = "";
    window.openDialog("chrome://mathmled/content/mathmledOp.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.val.length > 0)
      inserttag("mo",o.val);
  }
}

function doTimesCmd()
{
  var s = wind.getSelection();
  if (s)
    inserttimes();
}

function doDivideCmd()
{
  var s = wind.getSelection();
  if (s)
    insertdivide();
}

function doPlusminusCmd()
{
  var s = wind.getSelection();
  if (s)
    insertplusminus();
}

function doComposeCmd()
{
  var s = wind.getSelection();
  if (s)
    insertcompose();
}

function doCdotCmd()
{
  var s = wind.getSelection();
  if (s)
    insertcdot();
}

function doNablaCmd()
{
  var s = wind.getSelection();
  if (s)
    insertnabla();
}

function doBinOperatorCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.val = "";
    window.openDialog("chrome://mathmled/content/mathmledBinOp.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.val.length > 0)
      inserttag("mo",o.val);
  }
}

function doLECmd()
{
  var s = wind.getSelection();
  if (s)
    insertle();
}

function doNECmd()
{
  var s = wind.getSelection();
  if (s)
    insertne();
}

function doGECmd()
{
  var s = wind.getSelection();
  if (s)
    insertge();
}

function doElementCmd()
{
  var s = wind.getSelection();
  if (s)
    insertelement();
}

function doSubsetCmd()
{
  var s = wind.getSelection();
  if (s)
    insertsubset();
}

function doArrowCmd()
{
  var s = wind.getSelection();
  if (s)
    insertarrow();
}

function doParensCmd()
{
  var s = wind.getSelection();
  if (s)
    insertparens();
}

function doBracketsCmd()
{
  var s = wind.getSelection();
  if (s)
    insertbrackets();
}

function doBracesCmd()
{
  var s = wind.getSelection();
  if (s)
    insertbraces();
}

function doMatrixCmd(r,c)
{
  var s = wind.getSelection();
  if (s)
    insertmatrix(r,c);
}

function doAnyMatrixCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.rows = 1;
    o.cols = 1;
    window.openDialog("chrome://mathmled/content/mathmledmatrix.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.rows > 0)
      insertmatrix(o.rows,o.cols);
  }
}

function doSinCmd()
{
  var s = wind.getSelection();
  if (s)
    insertsin();
}

function doCosCmd()
{
  var s = wind.getSelection();
  if (s)
    insertcos();
}

function doTanCmd()
{
  var s = wind.getSelection();
  if (s)
    inserttan();
}

function doLnCmd()
{
  var s = wind.getSelection();
  if (s)
    insertln();
}

function doExpCmd()
{
  var s = wind.getSelection();
  if (s)
    insertexp();
}

function doLimCmd()
{
  var s = wind.getSelection();
  if (s)
    insertlim();
}

function doMathNameCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.val = "";
    window.openDialog("chrome://mathmled/content/mathmledMathName.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.val.length > 0)
      insertname(o.val);
  }
}

function doNolineCmd()
{
  var s = wind.getSelection();
  if (s)
    insertfractionnoline();
}

function doBevelledCmd()
{
  var s = wind.getSelection();
  if (s)
    insertfractionbevelled();
}

function doThickCmd()
{
  var s = wind.getSelection();
  if (s)
    insertfractionthick();
}

function doAboveCmd()
{
  var s = wind.getSelection();
  if (s)
    insertmover();
}

function doBelowCmd()
{
  var s = wind.getSelection();
  if (s)
    insertmunder();
}

function doAboveBelowCmd()
{
  var s = wind.getSelection();
  if (s)
    insertmunderover();
}

function doInfinityCmd()
{
  var s = wind.getSelection();
  if (s)
    insertinfinity();
}

function doPartialCmd()
{
  var s = wind.getSelection();
  if (s)
    insertpartial();
}

function doEllCmd()
{
  var s = wind.getSelection();
  if (s)
    insertell();
}

function doalphaCmd()
{
  var s = wind.getSelection();
  if (s)
    insertalpha();
}

function dopiCmd()
{
  var s = wind.getSelection();
  if (s)
    insertpi();
}

function dothetaCmd()
{
  var s = wind.getSelection();
  if (s)
    inserttheta();
}

function doGreekCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.val = "";
    window.openDialog("chrome://mathmled/content/mathmledGreek.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.val.length > 0)
      inserttag("mi",o.val);
  }
}

function doSpacingCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.val = "";
    window.openDialog("chrome://mathmled/content/mathmledSpace.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.val.length > 0)
      inserttag("mspace",o.val);
  }
}

function doInvisibleCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.val = "";
    window.openDialog("chrome://mathmled/content/mathmledInvisible.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.val.length > 0)
      inserttag("mo",o.val);
  }
}

function domiCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.val = "";
    window.openDialog("chrome://mathmled/content/mathmledmi.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.val.length > 0)
      inserttag("mi",o.val);
  }
}

function domoCmd()
{
  var s = wind.getSelection();
  if (s) {
    var o = new Object();
    o.val = "";
    window.openDialog("chrome://mathmled/content/mathmledmo.xul","_blank","chrome,close,titlebar,modal",o);
    if (o.val.length > 0)
      inserttag("mo",o.val);
  }
}

function doViewSourceCmd() {
  var s = wind.getSelection();
  if (s) {
    viewmathnode(s.focusNode);
  }
}

function doAttributesCmd() {
  var s = wind.getSelection();
  if (s) {
    viewattributes(s.focusNode);
  }
}

function doKey(evt)
{
  if (evt.altKey || evt.ctrlKey || evt.charCode == 0) {
    return;
  } else {
    switch (evt.charCode) {
    case 32:
      dospace();
      break;
    default:
      insertchar(evt.charCode);
      break;
    }
  }
  evt.preventBubble();
}
