/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla MathML Editor.
 *
 * The Initial Developer of the Original Code is
 * MacKichan Software, Inc.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Steve Swanson (steve.swanson@mackichan.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// mathmledAttributes.js - code to run the Edit Attributes dialog
//
// extensively copied from EdAdvancedEdit.js, etc.

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var bundle = srGetStrBundle("chrome://mathmled/locale/mathmled.properties");

var gDialog     = {};
var gElement    = null; // handle to actual element edited

var MathMLAttrs   = [];   // mathml attributes
var MathMLRAttrs  = [];   // removed mathml attributes

var gDoOnSelectTree = true;
var gUpdateTreeValue = true;


function Startup()
{
  // This is the return value for the parent,
  // who only needs to know if OK was clicked
  window.opener.AdvancedEditOK = false;

  // Element to edit is passed in
  if (!window.arguments[0])
  {
    dump("Attribute Edit: Element to edit not supplied\n");
    window.close();
    return;
  }
  // The actual element edited (not a copy!)
  gElement = window.arguments[0];

  // Create dialog object to store controls for easy access
  gDialog.AddHTMLAttributeNameInput     = document.getElementById("AddMathMLAttributeNameInput");
  gDialog.AddHTMLAttributeValueDeck     = document.getElementById("AddMathMLAttributeValueDeck");
  gDialog.AddHTMLAttributeValueMenulist = document.getElementById("AddMathMLAttributeValueMenulist");
  gDialog.AddHTMLAttributeValueTextbox  = document.getElementById("AddMathMLAttributeValueTextbox");
  gDialog.AddHTMLAttributeValueInput    = gDialog.AddHTMLAttributeValueTextbox;
  gDialog.AddHTMLAttributeTree          = document.getElementById("MathMLATree");

  Populate();

  window.sizeToContent();
}

function Populate()
{
  // place the tag name in the header
  var tagLabel = document.getElementById("tagLabel");
  tagLabel.setAttribute("value", ("<" + gElement.localName + ">"));

  BuildHTMLAttributeTable();
}

function onAccept()
{
  try {
    // Update our gElement attributes
    UpdateMathMLAttributes();
  } catch(ex) {
    dump(ex);
  }

  window.opener.AdvancedEditOK = true;
  return true;
}

function onCancel()
{
  return true;
}

function UpdateAndClear()
{
  try {
    // Update our gElement attributes
    UpdateMathMLAttributes();
  } catch(ex) {
    dump(ex);
  }
  ClearHTMLInputWidgets();
  ClearTreelist("MathMLAList");
}

function onParent()
{
  var n = gElement.parentNode;
  if (n)
  {
    UpdateAndClear();
    gElement = n;
    Populate();
  }
  else
  {
    alert(bundle.GetStringFromName("noParentAlert"));
  }
}

function onChild()
{
  var n = gElement.firstChild;
  if (n && n.nodeType == Node.TEXT_NODE)
    n = n.nextSibling
  if (n && n.nodeType != Node.TEXT_NODE)
  {
    UpdateAndClear();
    gElement = n;
    Populate();
  }
  else
  {
    alert(bundle.GetStringFromName("noChildAlert"));
  }
}

function onPrevious()
{
  var n = gElement.previousSibling;
  if (n && n.nodeType == Node.TEXT_NODE)
    n = n.previousSibling
  if (n && n.nodeType != Node.TEXT_NODE)
  {
    UpdateAndClear();
    gElement = n;
    Populate();
  }
  else
  {
    alert(bundle.GetStringFromName("noPreviousAlert"));
  }
}

function onNext()
{
  var n = gElement.nextSibling;
  if (n && n.nodeType == Node.TEXT_NODE)
    n = n.nextSibling;
  if (n && n.nodeType != Node.TEXT_NODE)
  {
    UpdateAndClear();
    gElement = n;
    Populate();
  }
  else
  {
    alert(bundle.GetStringFromName("noNextAlert"));
  }
}

// build attribute list in tree form from element attributes
function BuildHTMLAttributeTable()
{
  MathMLAttrs   = [];
  MathMLRAttrs  = [];
  var nodeMap = gElement.attributes;
  var i;
  if (nodeMap.length > 0) 
  {
    var added = false;
    for(i = 0; i < nodeMap.length; i++)
    {
      var name  = nodeMap[i].nodeName;
      if ( CheckAttributeNameSimilarity( name, MathMLAttrs ) ||
          IsEventHandler( name ) || IsMozInternal( name ) || IsStyleName( name ) ) {
        continue;   // repeated or non-MathML attribute, ignore this one and go to next
      }
      var value = gElement.getAttribute ( nodeMap[i].nodeName );
      if ( name.indexOf("_moz") != 0 &&
           AddTreeItem(name, value, "MathMLAList", MathMLAttrs) )
        added = true;
    }

    if (added)
      SelectHTMLTree(0);
  }
}

function CheckAttributeNameSimilarity(attName, attArray)
{
  for (var i = 0; i < attArray.length; i++)
  {
    if (attName == attArray[i])
      return true;
  }
  return false;
}

function IsEventHandler( which )
{
  var handlerName = which;
  var firstTwo = handlerName.substring(0,2);
  if (firstTwo == "on")
    return true;
  else
    return false;
}

function IsMozInternal( which )
{
  var handlerName = which;
  var firstFour = handlerName.substring(0,4);
  if (firstFour == "-moz")
    return true;
  else
    return false;
}

function IsStyleName( which )
{
  var handlerName = TrimString(which.toLowerCase());
  if (handlerName == "style")
    return true;
  else
    return false;
}

function RemoveMathMLAttribute()
{
  var treechildren = gDialog.AddHTMLAttributeTree.lastChild;

  // We only allow 1 selected item
  if (gDialog.AddHTMLAttributeTree.treeBoxObject.selection.count)
  {
    var item = getSelectedItem(gDialog.AddHTMLAttributeTree);
    var attr = GetTreeItemAttributeStr(item);

    // remove the item from the attribute array
    MathMLRAttrs[MathMLRAttrs.length] = attr;
    RemoveNameFromAttArray(attr, MathMLAttrs);

    // Remove the item from the tree
    treechildren.removeChild(item);

    // Clear inputs and selected item in tree
    ClearHTMLInputWidgets();
  }
}

function onInputMathMLAttributeName()
{
  var attName = TrimString(gDialog.AddHTMLAttributeNameInput.value);

  // Clear value widget, but prevent triggering update in tree
  gUpdateTreeValue = false;
  gDialog.AddHTMLAttributeValueInput.value = "";
  gUpdateTreeValue = true; 

  if (attName)
  {
    var newValue = "";
    // If attribute already exists in tree, use associated value,
    var existingValue = GetAndSelectExistingAttributeValue(attName, "MathMLAList");
    if (existingValue)
      newValue = existingValue;
      
    gDialog.AddHTMLAttributeValueInput.value = newValue;
  }
}

function onInputMathMLAttributeValue()
{
  if (!gUpdateTreeValue)
    return;

  var name = TrimString(gDialog.AddHTMLAttributeNameInput.value);
  if (!name)
    return;

  // Trim spaces only from left since we must allow spaces within the string
  //  (we always reset the input field's value below)
  var value = TrimStringLeft(gDialog.AddHTMLAttributeValueInput.value);
  if (value)
  {
    // Do value filtering based on type of attribute
    // (Do not use "LimitStringLength()" and "forceInteger()"
    //  to avoid multiple reseting of input's value and flickering)
    var selectedItem = gDialog.AddHTMLAttributeNameInput.selectedItem;

    if (selectedItem)
    {
      if ( selectedItem.getAttribute("forceOneChar") == "true" &&
           value.length > 1 )
        value = value.slice(0, 1);

      if ( selectedItem.getAttribute("forceIntOrPercent") == "true" )
      {
        // Allow integer with optional "%" as last character
        var percent = TrimStringRight(value).slice(-1);
        value = value.replace(/\D+/g,"");
        if (percent == "%")
          value += percent;
      }
      else if ( selectedItem.getAttribute("forceInteger") == "true" )
      {
        value = value.replace(/\D+/g,"");
      }
      else if ( selectedItem.getAttribute("forceSignedInteger") == "true" )
      {
        // Allow integer with optional "+" or "-" as first character
        var sign = value[0];
        value = value.replace(/\D+/g,"");
        if (sign == "+" || sign == "-")
          value = sign + value;
      }
      
      // Special case attributes 
      if (selectedItem.getAttribute("limitFirstChar") == "true")
      {
        // Limit first character to letter, and all others to 
        //  letters, numbers, and a few others
        value = value.replace(/^[^a-zA-Z\u0080-\uFFFF]/, "").replace(/[^a-zA-Z0-9_\.\-\:\u0080-\uFFFF]+/g,'');
      }

      // Update once only if it changed
      if (value != gDialog.AddHTMLAttributeValueInput.value)
        gDialog.AddHTMLAttributeValueInput.value = value;
    }
  }

  // Update value in the tree list
  // If not found, add new attribute
  if ( !UpdateExistingAttribute(name, value, "MathMLAList" ) && value)
    AddTreeItem(name, value, "MathMLAList", MathMLAttrs);
}

function UpdateExistingAttribute( attName, attValue, treeChildrenId )
{
  var treeChildren = document.getElementById(treeChildrenId);
  if (!treeChildren)
    return false;

  var name;
  var i;
  attName  = TrimString(attName);
  attValue = TrimString(attValue);

  for (i = 0; i < treeChildren.childNodes.length; i++)
  {
    var item = treeChildren.childNodes[i];
    name = GetTreeItemAttributeStr(item);
    if (name == attName)
    {
      // Set the text in the "value' column treecell
      SetTreeItemValueStr(item, attValue);

      // Select item just changed, 
      //  but don't trigger the tree's onSelect handler
      gDoOnSelectTree = false;
      try {
        selectTreeItem(treeChildren, item);
      } catch (e) {}
      gDoOnSelectTree = true;

      return true;
    }
  }
  return false;
}

function GetAndSelectExistingAttributeValue( attName, treeChildrenId )
{
  if (!attName)
    return "";

  var treeChildren = document.getElementById(treeChildrenId);
  var name;
  var i;

  for (i = 0; i < treeChildren.childNodes.length; i++)
  {
    var item = treeChildren.childNodes[i];
    name = GetTreeItemAttributeStr(item);
    if (name == attName)
    {
      // Select item in the tree
      //  but don't trigger the tree's onSelect handler
      gDoOnSelectTree = false;
      try {
        selectTreeItem(treeChildren, item);
      } catch (e) {}
      gDoOnSelectTree = true;

      // Get the text in the "value' column treecell
      return GetTreeItemValueStr(item);
    }
  }

  // Attribute doesn't exist in tree, so remove selection
  gDoOnSelectTree = false;
  try {
    treeChildren.parentNode.treeBoxObject.selection.clearSelection();
  } catch (e) {}
  gDoOnSelectTree = true;

  return "";
}

// update the object with added and removed attributes
function UpdateMathMLAttributes()
{
  var MathMLAList = document.getElementById("MathMLAList");
  var i;

  // remove removed attributes
  for (i = 0; i < MathMLRAttrs.length; i++)
  {
    var name = MathMLRAttrs[i];

    if (gElement.hasAttribute(name))
      doRemoveAttribute(name);
  }

  // Set added or changed attributes
  for( i = 0; i < MathMLAList.childNodes.length; i++)
  {
    var item = MathMLAList.childNodes[i];
    if (IsMozInternal(GetTreeItemAttributeStr(item)))  // don't write these out
      continue;
    doSetAttribute( GetTreeItemAttributeStr(item), GetTreeItemValueStr(item));
  }
}

function RemoveHTMLAttribute()
{
  var treechildren = gDialog.AddHTMLAttributeTree.lastChild;

  // We only allow 1 selected item
  if (gDialog.AddHTMLAttributeTree.treeBoxObject.selection.count)
  {
    var item = getSelectedItem(gDialog.AddHTMLAttributeTree);
    var attr = GetTreeItemAttributeStr(item);

    // remove the item from the attribute array
    MathMLRAttrs[MathMLRAttrs.length] = attr;
    RemoveNameFromAttArray(attr, MathMLAttrs);

    // Remove the item from the tree
    treechildren.removeChild(item);

    // Clear inputs and selected item in tree
    ClearHTMLInputWidgets();
  }
}

function SelectHTMLTree( index )
{

  gDoOnSelectTree = false;
  try {
    gDialog.AddHTMLAttributeTree.selectedIndex = index;
  } catch (e) {}
  gDoOnSelectTree = true;
}

function ClearHTMLInputWidgets()
{
  gDialog.AddHTMLAttributeTree.treeBoxObject.selection.clearSelection();
  gDialog.AddHTMLAttributeNameInput.value  = "";
  gDialog.AddHTMLAttributeValueInput.value = "";
  SetTextboxFocus(gDialog.AddHTMLAttributeNameInput);
}

function SetTextboxFocus(textbox)
{
  if (textbox)
  {
    setTimeout( function(textbox) { textbox.focus(); }, 0, textbox );
  }
}

function doRemoveAttribute(attrib)
{
  gElement.removeAttribute(attrib);
}

function doSetAttribute(attrib, value)
{
  gElement.setAttribute(attrib, value);
}

function RemoveNameFromAttArray(attName, attArray)
{
  for (var i=0; i < attArray.length; i++)
  {
    if (attName == attArray[i])
    {
      // Remove 1 array item
      attArray.splice(i,1);
      break;
    }
  }
}


// adds a generalised treeitem.
function AddTreeItem ( name, value, treeChildrenId, attArray )
{
  attArray[attArray.length] = name;
  var treeChildren    = document.getElementById ( treeChildrenId );
  var treeitem    = document.createElementNS ( XUL_NS, "treeitem" );
  var treerow     = document.createElementNS ( XUL_NS, "treerow" );

  var attrCell    = document.createElementNS ( XUL_NS, "treecell" );
  attrCell.setAttribute( "class", "propertylist" );
  attrCell.setAttribute( "label", name );

  var valueCell    = document.createElementNS ( XUL_NS, "treecell" );
  valueCell.setAttribute( "class", "propertylist" );
  valueCell.setAttribute( "label", value );

  treerow.appendChild ( attrCell );
  treerow.appendChild ( valueCell );
  treeitem.appendChild ( treerow );
  treeChildren.appendChild ( treeitem );

  // Select item just added,
  //  but suppress calling the onSelect handler
  gDoOnSelectTree = false;
  try {
    treeChildren.parentNode.selectItem(treeitem);
  } catch (e) {}
  gDoOnSelectTree = true;

  return treeitem;
}

function GetTreeItemAttributeStr(treeItem)
{
  if (treeItem)
    return TrimString(treeItem.firstChild.firstChild.getAttribute("label"));

  return "";
}

function GetTreeItemValueStr(treeItem)
{
  if (treeItem)
    return TrimString(treeItem.firstChild.lastChild.getAttribute("label"));

  return "";
}

function SetTreeItemValueStr(treeItem, value)
{
  if (treeItem && GetTreeItemValueStr(treeItem) != value)
    treeItem.firstChild.lastChild.setAttribute("label", value);
}

function TrimStringLeft(string)
{
  if(!string) return "";
  return string.replace(/^\s+/, "");
}

function TrimString(string)
{
  if (!string) return "";
  return string.replace(/(^\s+)|(\s+$)/g, '')
}

function onSelectMathMLTreeItem()
{
  if (!gDoOnSelectTree)
    return;

  var tree = gDialog.AddHTMLAttributeTree;
  if (tree && tree.treeBoxObject.selection.count)
  {
    var inputName = TrimString(gDialog.AddHTMLAttributeNameInput.value);
    var selectedItem = getSelectedItem(tree);
    var selectedName = selectedItem.firstChild.firstChild.getAttribute("label");

    if (inputName == selectedName)
    {
      // Already editing selected name - just update the value input
      gDialog.AddHTMLAttributeValueInput.value = GetTreeItemValueStr(selectedItem);
    }
    else
    {
      gDialog.AddHTMLAttributeNameInput.value = selectedName;

      // Change value input based on new selected name
      onInputHTMLAttributeName();
    }
  }
}

function editMathMLAttributeValue(targetCell)
{
  if (IsNotTreeHeader(targetCell))
    gDialog.AddHTMLAttributeValueInput.inputField.select();
}

function IsNotTreeHeader(treeCell)
{
  if (treeCell)
    return (treeCell.parentNode.parentNode.nodeName != "treehead");

  return false;
}

function getSelectedItem(tree)
{
  if (tree.treeBoxObject.selection.count == 1)
    return tree.contentView.getItemAtIndex(tree.currentIndex);
  else
    return null;
}

function ClearTreelist(treeChildrenId)
{
  var treeChildren = document.getElementById(treeChildrenId);
  if (!treeChildren)
    return;

  gDoOnSelectTree = false;
  try {
    treeChildren.parentNode.treeBoxObject.selection.clearSelection();
  } catch (e) {}
  gDoOnSelectTree = true;

  if (treeChildren.firstChild)
  {
    var nextChild = treeChildren.firstChild;
    while (nextChild)
    {
      var nextTmp = nextChild.nextSibling;
      treeChildren.removeChild(nextChild);
      nextChild = nextTmp;
    }
  }
}

