/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla MathML Editor.
 *
 * The Initial Developer of the Original Code is
 * MacKichan Software, Inc.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Steve Swanson (steve.swanson@mackichan.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// This is a modified version of viewsource.js.  That file, however,
// was not available in Moz1.0 and was not general enough.

var gDebug = 0;
var gLineCount = 0;
var gStartTargetLine = 0;
var gEndTargetLine = 0;
var gTargetNode = null;

var gViewSourceCSS = 'chrome://mathmled/content/viewsource.css';

function viewmath(evt) {
  evt.preventBubble();
  viewmathnode(evt.target);
}

function viewmathnode(node) {
  gLineCount = 0;
  gStartTargetLine = 0;
  gEndTargetLine = 0;
  gTargetNode = node;
  if (gTargetNode.nodeType == Node.TEXT_NODE) { // leaf text
    gTargetNode = gTargetNode.parentNode;
  }
  var m = findmathparent(node);
  if (m) {
    var markup = open("", "wysiwyg", "scrollbars=1,resizable=1,width=500,height=200");
    markup.document.open();
    with (markup.document) {
      writeln('<html><base target="wysiwyg">');
      writeln('<head><title>MathML WYSIWYG Source</title>');
      writeln('<style>#target{border:dashed 1px; background-color:lightyellow;}</style>'); 
      writeln('<script>');
      writeln('var repeatCount = 10; var opacity = 0; var sign = +1;');
      writeln('function scrollToTarget() {');
                 // bug: self.location.hash = 'anchor'; doesn't work
      writeln('  var anchor = document.anchors["target"];');
      writeln('  scrollTo(0/*anchor.offsetLeft*/, anchor.offsetTop);');
      writeln('}</script>');
      writeln('<link rel="stylesheet" type="text/css" href="'+gViewSourceCSS+'">');
      writeln('</head><body onload="window.focus();scrollToTarget();" bgcolor="#FFFFFF">');
      writeln('<pre class="viewsource">');
      writeln(getOuterMarkup(m,0,true));
      writeln('</pre></body></html>');
    }
    markup.document.close();
  }
}

function getDocMarkup(doc) {
  gLineCount       = 0;
  gStartTargetLine = 0;
  gEndTargetLine   = 0;
  gTargetNode      = 0;
  return(getOuterMarkup(doc,-2,false));
}

function getInnerMarkup(n, indent, addMarkup) {
  var str = '';
  for (var i = 0; i<n.childNodes.length; i++)
    str += getOuterMarkup(n.childNodes[i],indent,addMarkup);
  return str;
}

function getOuterMarkup(n, indent, addMarkup) {
  var newline = '';
  var padding = '';
  var str = '';
  if (n == gTargetNode) {
    gStartTargetLine = gLineCount;
    if (addMarkup);
      str += '<a name="target"></a><div id="target">';
  }

  switch (n.nodeType) {
  case Node.ELEMENT_NODE: // Element
    // to avoid the wide gap problem, '\n' is not emitted on the first
    // line and the lines before & after the <div id="target">...</div>
    newline = '';
    if (gLineCount > 0) {
      newline = '\n';
    }
    if (gLineCount == gStartTargetLine || gLineCount == gEndTargetLine) {
      newline = '';
    }
    gLineCount++;
    if (gDebug) {
      newline += gLineCount;
    }
    for (var k=0; k<indent && indent>0; k++) {
      padding += ' ';
    }
    str += newline + padding;
    if (addMarkup)
      str += '&lt;<span class="start-tag">' + n.nodeName + '</span>';
    else
      str += '<' + n.nodeName;
    for (var i=0; i<n.attributes.length; i++) {
      var attr = n.attributes[i];
      if (attr.nodeName.match(/^-moz-math-/)) {
        if (!gDebug) {
          continue;
        }
      }
      if (addMarkup) {
        str += ' <span class="attribute-name">';
        str += attr.nodeName;
        str += '</span>=<span class="attribute-value">"';
        str += attr.nodeValue;
        str += '"</span>';
      } else {
        str += ' ' + attr.nodeName + '="' + attr.nodeValue + '"';
      }
    }
    if (addMarkup)
      str += '&gt;';
    else
      str += '>';
    var oldLine = gLineCount;
    str += getInnerMarkup(n, indent + 2, addMarkup);
    if (oldLine == gLineCount) {
      newline = '';
      padding = '';
    }
    else {
      newline = '\n';
      if (gLineCount == gEndTargetLine) {
        newline = '';
      }
      gLineCount++;
      if (gDebug) {
        newline += gLineCount;
      }
    }
    str += newline + padding;
    if (addMarkup)
      str += '&lt;/<span class="end-tag">' + n.nodeName + '</span>&gt;';
    else
      str += '</' + n.nodeName + '>';
    break;
  case Node.TEXT_NODE: // Text
    var nodetext = n.nodeValue;
    nodetext = nodetext.toString();
    var outtext;
    if (addMarkup) {
	    outtext = nodetext.replace(/(\n|\r|\t)+/g, " ");
	    outtext = outtext.replace(/^ +/, "");
	    outtext = outtext.replace(/ +$/,"");
	    if (outtext.length != 0) {
	      str += '<span class="text">' + unicode2entity(outtext,true) + '</span>';
      }
    } else {
      outtext = nodetext.replace(/[\t\n\r ]+/g, " ");
      str += unicode2entity(outtext,false);
	  }
    break;
  case Node.DOCUMENT_NODE:
    // skip over
    str += getInnerMarkup(n,indent,addMarkup);
    break;
  case Node.DOCUMENT_TYPE_NODE:
    // ignore
    break;
  case Node.COMMENT_NODE:
    if (addMarkup)
      str += '<span class="comment">&lt;!--';
    else
      str += '<!--';
	  str += n.nodeValue;
    if (addMarkup)
      str += '--&gt;</span>';
    else
      str += '-->\n';
    break;
  default:
    alert("Unknown nodeType: " + n.nodeType);
    break;
  }

  if (n == gTargetNode) {
    gEndTargetLine = gLineCount;
    str += '</div>';
  }
  return str;
}

//    case Node.ELEMENT_NODE:                 return "Element";
//    case Node.ATTRIBUTE_NODE:               return "Attribute";
//    case Node.TEXT_NODE:                    return "Text";
//    case Node.CDATA_SECTION_NODE:           return "CDATA Section";
//    case Node.ENTITY_REFERENCE_NODE:        return "Entity Reference";
//    case Node.ENTITY_NODE:                  return "Entity";
//    case Node.PROCESSING_INSTRUCTION_NODE:  return "Processing Instruction";
//    case Node.COMMENT_NODE:                 return "Comment";
//    case Node.DOCUMENT_NODE:                return "Document";
//    case Node.DOCUMENT_TYPE_NODE:           return "Document Type";
//    case Node.DOCUMENT_FRAGMENT_NODE:       return "Document Fragment";
//    case Node.NOTATION_NODE:                return "Notation";
